/*                    wilitfit.c
  routine to support batch file test of file fit to disk	
  returns errorlevel = 5 if file won't fit on specified drive, 
  exit errorlevel = 4 if error
  exit errorlevel = 3 if DOS critical error (Abort, Retry,...?)
  else errorlevel = 0
*/

/*Example of use (test batch program)

echo off
wilitfit %1 %2
if errorlevel 5 goto no
if errorlevel 4 goto err
if errorlevel 3 goto abo
if errorlevel 0 goto ok
:ok
echo It fits
goto end
:no
echo It won't fit
goto end
:err
echo Error in args
goto end
:abo
echo Abort, disk error
goto end
:end

Note:  The first argument (drive) must be a letter, else errorlevel = 4;
		 If the drive doesn't exist or ditto the file, then errorlevel = 4;
		 If not 2 args or more then errorlevel = 4;
		 If the filename doesn't begin with an alpha, errorlevel = 4;
		 If file doesn't exist, errorlevel = 4;
		 If an DOS disk error(Abort,Retry,...?) is received, errorlevel = 3;
		 If the above OK, but file bigger than disk space errorlevel = 5;
		 If none of the above, errorlevel = 0;
		 No protect for self overwrite disasters!, i.e. no warning of same
		 file existing on target drive(which could be source drive, too).
		 DOS copy helps on this.
		 No Check here for write enable on destination.

		 Compiled in TC++ 1.0, 2 Dec 90, tested with above routine from keyboard
		 input, compact model used for compile.

		 Released to public domain 2Dec90. O. H. Jackson.  From here on it's
		 your problem only!

*/

/*Define DEBUG here for test prints*/

#ifdef DEBUG
#define D(x) x
#endif
#ifndef DEBUG
#define D(x)  
#endif


#include <stdio.h>
#include <dir.h>
#include <ctype.h>
#include <dos.h>
#include <stdlib.h>
#include <signal.h>
/*
	call with disk letter or full file/path name in ist 1st arg, existing 
	filename in 2nd
*/

/*Prototypes*/
void catcher(void);    /*Signal responder*/
/*critical error handler*/
void interrupt raiser(void);
void usage(void);

/************************* main ************************************/

void main(int argc,char **argv)
{

	int flags, ffirst, ferr;
	long diskcap = 0L;
	long filsiz = 1500000L;
	union REGS ireg, oreg;
	struct SREGS segreg;
	struct ffblk d;
/*	void interrupt (*old)(void);*/  /*not needed for .EXE */

	/*set up mechanism to trap disk errors   */
	signal(SIGABRT,catcher);  /*Exits on level 3 error*/

	/*tie us to crit error handler in case disk,etc. fail*/
	/*this will raise SIGABRT on disk error*/
	/*instead of Abort, Retry, Ignore? */
	/* then catcher() will exit at errorlevel 3 -- it's magic*/
/*	old = getvect(0x24);*/          /*not needed for .EXE */
	setvect(0x24,raiser); /*raiser() below is new interrupt service routine (isr)*/
	/*interrupt setup done*/

	/*now do the sizing*/
	/*check input data first*/
	if(argc < 3)
	{
		usage();
		exit(4);   /*won't fit--doesn't exist*/
	}
	D(fprintf(stderr,"\nDrive = %s, File = %s\n",argv[1],argv[2]);)

	flags = !isalnum(argv[2][0]);
	if(flags)     /*assumes a * or ? instead of a letter or digit*/
		exit(4);   /*single files only*/

/* get drive space*/
	ireg.h.ah = 0x36;  /*drive allocation info*/
	ireg.h.dl = toupper(argv[1][0]) - 'A' + 1;  /*no default allowed-must be letter*/
	intdos(&ireg, &oreg);
	if(0xffff == oreg.x.ax)
		exit(4);   /*can't find drive*/

	/*cap = sects/clust * avail clust * bytes/sector*/
	diskcap = (long) oreg.x.ax * (long) oreg.x.bx * (long) oreg.x.cx;

/*get file size*/
	ffirst = findfirst(argv[2], &d, 0);
	if (-1 == ffirst) 
		exit(4);
	filsiz = d.ff_fsize;

	D(fprintf(stderr,"Drive left = %ld, filsiz = %ld\n",diskcap,filsiz);)

	if(filsiz > diskcap)
		exit(5);    /*really won't fit*/
	else
		exit(0);    /*no problems*/
}
/*note that errorlevel 3 comes from catcher, below*/

/*************************** catcher **********************************/
void catcher(void)
{
	D(puts("\nExiting from Catcher\n");)
	exit(3);
}

/************************* raiser ***********************************/                                                                        
void interrupt raiser(void)
{
	raise(SIGABRT);      /*signal to raise if critical error occurs*/
}

/***************************** usage ********************************/
void usage(void)
{
	fputs("\n Error...Usage: wilitfit drive_letter filename\n",stderr);
} 
/*************************** end ************************************/